module gauss_bd

 use gauss

 implicit none
 integer :: ng_bd,ngf_bd
 double precision, allocatable :: xi_bd(:),yi_bd(:),wi_bd(:)
 double precision, allocatable :: wif_bd(:),tif_bd(:)
 double precision, allocatable :: pi2_bd(:,:),pico_bd(:,:)
 
 contains

!
!
! --------------------------------------- 
!    read the gauss files
! --------------------------------------- 
!
 
subroutine read_gauss_bd
      implicit none
      character*30 fng,fngf
      integer ideb,ifin
      character name*1,anum*3
      integer i
      character*60 :: format1,format2
     
      allocate(xi_bd(ng_bd),yi_bd(ng_bd),wi_bd(ng_bd))
      allocate(wif_bd(ngf_bd),tif_bd(ngf_bd))
!
!-----------------------------------
! define name files to read
!
      write(anum,'i3') ng_bd
      ideb=1
      do while (anum(ideb:ideb).eq.' ') 
         ideb=ideb+1
      enddo
      fng  = 'include/tri2_'//anum(ideb:3)//'.dat'

      write(anum,'i3') ngf_bd
      ideb=1
      do while (anum(ideb:ideb).eq.' ') 
         ideb=ideb+1
      enddo
      fngf = 'include/legen'//anum(ideb:3)//'.dat'

!-----------------------------------
! read files
!
      open(1,file=fng,status='old')
      do i=1,ng_bd
         read(1,*) wi_bd(i),xi_bd(i),yi_bd(i)
      enddo
      close(1)
      open(1,file=fngf,status='old')
      do i=1,ngf_bd
         read(1,*) wif_bd(i),tif_bd(i)
      enddo
      close(1)

!------------------------------------
! write messages to standart output
!
!
format1='1x,60(''-'')'
format2='(1x,a21,i7,3x,'

      write(*,*)
      write(*,format1)
      write(*,*) 'READ GAUSSIAN FILES FOR CURVED ELEMENTS'
      write(*,format1)
      write(*,*)
      write(*,*) 'load 2d Gauss points on triangle: ',fng
      write(*,*) 'load 1d Gauss points: ',fngf

      write(*,format2) 'tri gauss points    :',ng_bd
      write(*,format2) '1d gauss points     :',ngf_bd

      return

end subroutine read_gauss_bd

!
!**************************************************
! compute polynomial value at Gauss point locations
!**************************************************
!
subroutine cal_p2_bd

 implicit none
 double precision po,x,y
 integer i,j,kc1,kc2,i1
 external po

      allocate(pi2_bd(ng_bd,nm))

      i1 = 0
      do kc1=0,nc
	   do kc2=0,nc-kc1
	      i1 = i1 + 1
	      do i=1,ng_bd
	         y = yi_bd(i)
	         x = xi_bd(i)
	         pi2_bd(i,i1)=po(kc1,x)*po(kc2,y)
		enddo
	   enddo
      enddo
      return
end subroutine cal_p2_bd

!
!**************************************************************
! computes the value of the polynomes along the boundaries of 
! the element
!**************************************************************
!
subroutine cal_pico_bd

      implicit none
      double precision po,x,y
      external po
      integer i1,kc1,kc2,i,j
      
      allocate(pico_bd(ngf_bd,nm))

	i1=0
	do kc1=0,nc
	   do kc2=0,nc-kc1
	    i1=i1+1
	    do j=1,ngf_bd
	       x = tif_bd(j)
	       y = -1.d0
	       pico_bd(j,i1) = po(kc1,x)*po(kc2,y)
	    enddo
	   enddo
	enddo

	return
end subroutine cal_pico_bd

end module gauss_bd
